﻿using System;
using System.Drawing;
using System.Windows.Forms;
using BlockMania.GeneralStates.Screen_03_Settings;
using BlockMania.Global;
using BlockMania.Util;

namespace BlockMania
{
	public partial class FormBlockMania : Form
	{
		private Panel panelCenter;

		public FormBlockMania()
		{
			InitializeComponent();
			InitGUI();
		}

		private void InitGUI()
		{
			SuspendLayout();

			InitWindow();
			Panel panel = CreatePanel();
			Controls.Add(panel);

			ResumeLayout(false);
		}

		private void InitWindow()
		{
			Text = GlobalData.WINDOW_TITLE;
			BackColor = GlobalColors.WINDOW_COLOR;

			ClientSize = new Size(GlobalData.WINDOW_WIDTH, GlobalData.WINDOW_HEIGHT);
			MinimumSize = new Size(GlobalData.WINDOW_WIDTH, GlobalData.WINDOW_HEIGHT);

			AutoScaleMode = AutoScaleMode.None;

			FormClosing += FormBlockMania_FormClosing;
			Shown += FormBlockMania_Shown;
			KeyUp += FormBlockMania_KeyUp;
		}

		private Panel CreatePanel()
		{
			panelCenter = new Panel();
			panelCenter.Name = "panelCenter";
			panelCenter.Size = new Size(GlobalData.PANEL_WIDTH, GlobalData.PANEL_HEIGHT);
			panelCenter.Anchor = AnchorStyles.None;
			panelCenter.BackColor = GlobalColors.PANEL_COLOR;
			panelCenter.Location = new Point(10, 10);
			panelCenter.Margin = new Padding(0);
			panelCenter.TabIndex = 0;
			return panelCenter;
		}

		private void FormBlockMania_Shown(object sender, EventArgs e)
    {
			GlobalController.Init(panelCenter);
    }

    private static void FormBlockMania_FormClosing(object sender, FormClosingEventArgs e)
    {
	    GameLoopTimer.Stop();
	    SettingStorage.SaveSettings();
    }

    protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
    {
	    switch (keyData)
	    {
		    case Keys.Left:
		    case Keys.A:
		    {
			    GlobalController.SetUserInput(UserInput.MOVE_LEFT);
			    return true;
		    }
		    case Keys.Right:
		    case Keys.D:
		    {
			    GlobalController.SetUserInput(UserInput.MOVE_RIGHT);
			    return true;
		    }
		    case Keys.Up:
		    case Keys.W:
		    {
			    GlobalController.SetUserInput(UserInput.ROTATE);
			    return true;
		    }
		    case Keys.Down:
		    case Keys.S:
		    {
			    GlobalController.SetUserInput(UserInput.MOVE_DOWN);
			    return true;
		    }
		    case Keys.Enter:
		    case Keys.Space:
		    {
			    GlobalController.SetUserInput(UserInput.FLIP);
			    return true;
		    }
		    case Keys.Escape:
		    {
			    GlobalController.SetUserInput(UserInput.ESCAPE);
			    return true;
		    }
		    case Keys.F1:
		    {
			    GlobalController.SetUserInput(UserInput.SWITCH_TO_PLAY);
			    return true;
		    }
		    case Keys.F2:
		    {
			    GlobalController.SetUserInput(UserInput.SWITCH_TO_HOME);
			    return true;
		    }
		    case Keys.F3:
		    {
			    GlobalController.SetUserInput(UserInput.SWITCH_TO_SETTINGS);
			    return true;
		    }
		    case Keys.F4:
		    {
			    GlobalController.SetUserInput(UserInput.SWITCH_TO_HELP);
			    return true;
		    }
		    case Keys.T:
		    {
			    GlobalController.SetUserInput(UserInput.SELECT_BLOCK);
			    return true;
		    }
		    case Keys.F5:
		    {
			    GlobalController.SetUserInput(UserInput.F5);
			    return true;
		    }
		    case Keys.F6:
		    {
			    GlobalController.SetUserInput(UserInput.F6);
			    return true;
		    }
		    case Keys.F7:
		    {
			    GlobalController.SetUserInput(UserInput.F7);
			    return true;
		    }
		    case Keys.F8:
		    {
			    GlobalController.SetUserInput(UserInput.F8);
			    return true;
		    }
		    case Keys.F9:
		    {
			    GlobalController.SetUserInput(UserInput.F9);
			    return true;
		    }
		    default:
		    {
			    return base.ProcessCmdKey(ref msg, keyData);
		    }
	    }
    }

    private static void FormBlockMania_KeyUp(object sender, KeyEventArgs e)
    {
	    GlobalController.SetUserInput(UserInput.NONE);
    }

	}
}