﻿using System.Drawing;
using BlockMania.Global;
using BlockMania.Util;

namespace BlockMania.GeneralStates.Screen_01_Home
{
	internal static class Screen_1_Home
	{
		private enum ScreenState
		{
			REGULAR,
			OUTRO
		}
		private static ScreenState screenState = ScreenState.REGULAR;

		private static TextStatic textBlockMania;
		private static TextBlinking textStart;
		private static TextStatic textSettings;
		private static TextStatic textHelp;

		internal static void Init()
		{
			string text = GlobalData.WINDOW_TITLE;
			Font font   = FontManager.font_150;
			int Y = 90;
			int X = Tools.CalculateXforStringOnPanel(text, font);
			textBlockMania = new TextStatic(text, font, GlobalColors.BRUSH_YELLOW, X, Y);

			text = "press 'F1' to start";
			font = FontManager.font_65;
			Y = 350;
			X = Tools.CalculateXforStringOnPanel(text, font);
			textStart = new TextBlinking(text, font, GlobalColors.BRUSH_WHITE, 1000, X, Y);

			text = "settings : F3";
			font = FontManager.font_40;
			Y = GlobalData.PANEL_HEIGHT - 60;
			X = 20;
			textSettings = new TextStatic(text, font, GlobalColors.BRUSH_PINK, X, Y);

			text = "help : F4";
			font = FontManager.font_40;
			X = 795;
			textHelp = new TextStatic(text, font, GlobalColors.BRUSH_PINK, X, Y);
		}

		internal static void Reset()
		{
			textStart.Reset();
		}

		internal static void HandleGameTick(UserInput input, bool isNotRepeating)
		{
			Draw();

			switch (screenState)
			{
				case ScreenState.REGULAR : HandleInput(input, isNotRepeating) ; break;
				case ScreenState.OUTRO   : Transition.TriggerPartOne() ; break;
			}
		}

		private static void HandleInput(UserInput input, bool isNotRepeating)
		{
			if (input != UserInput.NONE && isNotRepeating)
			{
				if (input == UserInput.SWITCH_TO_PLAY)
				{
					SwitchToOutroState();
				}
			}
		}

		private static void SwitchToOutroState()
		{
			screenState = ScreenState.OUTRO;
		}

		internal static void SwitchToNormalState()
		{
			screenState = ScreenState.REGULAR;
		}

		private static void Draw()
		{
			textBlockMania.HandleGameTick();

			if (screenState == ScreenState.REGULAR)
				textStart.HandleGameTick();

			textSettings.HandleGameTick();
			textHelp.HandleGameTick();
		}

	}
}