﻿using System.Drawing;
using BlockMania.GeneralStates.Screen_03_Settings;
using BlockMania.Global;
using BlockMania.Util;

namespace BlockMania.GeneralStates.Screen_02_Play
{
	internal static class ScoreManager
	{
		private static int hiScore;
		private static int score;

		private static int rows;
		private static int rowsTemp;

		private static TextStatic textHiScoreLabel;
		private static TextStatic textHiScoreNumber;

		private static TextStatic textScoreLabel;
		private static TextStatic textScoreNumber;

		private static TextStatic textRowsLabel;
		private static TextStatic textRowsNumber;

		private static TextStatic textLevelLabel;
		private static TextStatic textLevelNumber;

		internal static void Init()
		{
			const int X = GlobalData.SCORE_LOCATION_X - 15;
			int Y = GlobalData.SCORE_LOCATION_Y;

			Font fontLabel  = FontManager.font_40;
			Font fontNumber = FontManager.font_50;

			Brush brushLabel = GlobalColors.BRUSH_PINK;
			Brush brushNumber = GlobalColors.BRUSH_YELLOW;

			hiScore = SettingStorage.hiScore;

			textHiScoreLabel = new TextStatic("hi-score", fontLabel, brushLabel, X, Y);
			Y += 35;
			textHiScoreNumber = new TextStatic($"{0:0000000}", fontNumber, brushNumber, X, Y);

			Y += 85;
			textScoreLabel = new TextStatic("score", fontLabel, brushLabel, X, Y);
			Y += 35;
			textScoreNumber = new TextStatic($"{0:0000000}", fontNumber, brushNumber, X, Y);

			Y += 85;
			textRowsLabel = new TextStatic("rows", fontLabel, brushLabel, X, Y);
			Y += 35;
			textRowsNumber = new TextStatic($"{0:000}", fontNumber, brushNumber, X, Y);

			Y += 85;
			textLevelLabel = new TextStatic("level", fontLabel, brushLabel, X, Y);
			Y += 35;
			textLevelNumber = new TextStatic($"{0:00}", fontNumber, brushNumber, X, Y);
		}

		internal static void Reset()
		{
			score    = 0;
			rows     = 0;
			rowsTemp = 0;

		}

		internal static void AddRows(int rowCount)
		{
			rows += rowCount;
			UpdateScore(rowCount);

			rowsTemp += rowCount;
			LevelUpIfAppropriate();
		}

		private static void UpdateScore(int rowCount)
		{
			 // The points for [cleared rows] are calculated as follows :
			 // points = (40 * level) * 1 or 2.5 or 7.5 or 30

			 double factor = 1;
			 switch (rowCount)
			 {
				 case 2: factor = 2.5 ; break;
				 case 3: factor = 7.5 ; break;
				 case 4: factor = 30  ; break;
			 }

			 int points = (int) (40 * BlockSpeed.speed * factor);
			 score += points;

			 if (score > hiScore)
			 {
				 hiScore = score;
				 SettingStorage.hiScoreNew = hiScore;
			 }
		}

		private static void LevelUpIfAppropriate()
		{
			if (SettingStorage.levelUp && rowsTemp >= GlobalData.LEVEL_UP_THRESHOLD)
			{
				BlockSpeed.Increase();
				rowsTemp -= GlobalData.LEVEL_UP_THRESHOLD;
			}
		}

		internal static void Render()
		{
			UpdateLabels();

			textHiScoreLabel.HandleGameTick();
			textHiScoreNumber.HandleGameTick();

			textScoreLabel.HandleGameTick();
			textScoreNumber.HandleGameTick();

			textRowsLabel.HandleGameTick();
			textRowsNumber.HandleGameTick();

			textLevelLabel.HandleGameTick();
			textLevelNumber.HandleGameTick();
		}

		private static void UpdateLabels()
		{
			textHiScoreNumber.text = $"{hiScore:0000000}";
			textScoreNumber.text   = $"{score:0000000}";
			textRowsNumber.text    = $"{rows:000}";
			textLevelNumber.text   = $"{BlockSpeed.speed:00}";
		}

	}
}