﻿using BlockMania.GeneralStates.Screen_01_Home;
using BlockMania.Global;
using BlockMania.Util;
using BlockMania.View;

namespace BlockMania.GeneralStates.Screen_02_Play
{
	internal static class Screen_2_Play
	{
		private static readonly RepetitionChecker repetitionChecker = new RepetitionChecker();

		private static bool isGamePaused;

		private enum PlayState
		{
			INTRO,
			REGULAR,
			ROW_REMOVAL,
			PAUSED,
			GAME_OVER
		}
		private static PlayState playState;

		internal static void Init()
		{
			State_02_Row_Removal.Init();
			State_03_Paused.Init();
			State_04_Game_Over.Init();
		}

		internal static void Reset()
		{
			Gamefield.Reset();
			ScoreManager.Reset();
			BlockSpeed.Reset();

			playState    = PlayState.INTRO;
			isGamePaused = false;
		}

		internal static void TogglePause()
		{
			isGamePaused = ! isGamePaused;
			State_03_Paused.Reset();
		}

		internal static void SwitchToNormalState()
		{
			playState = PlayState.REGULAR;
		}

		internal static void HandleGameTick(UserInput input, bool isNotRepeating)
		{
			PreviewField.Render();
			Gamefield.Render();
			ScoreManager.Render();

			DistributeGameTick(input, isNotRepeating);
		}

		private static void DistributeGameTick(UserInput input, bool isNotRepeating)
		{
			if (isGamePaused)
			{
				State_03_Paused.HandleGameTick();
			}
			else
			{
				switch (playState)
				{
					case PlayState.INTRO       : Transition.TriggerPartTwo()                              ; break;
					case PlayState.REGULAR     : State_01_Regular.HandleGameTick(input, isNotRepeating)   ; break;
					case PlayState.ROW_REMOVAL : State_02_Row_Removal.HandleGameTick()                    ; break;
					case PlayState.PAUSED      : State_03_Paused.HandleGameTick()                         ; break;
					case PlayState.GAME_OVER   : State_04_Game_Over.HandleGameTick(input, isNotRepeating) ; break;
				}
			}
		}

		internal static void SwitchToRegularGamePlay()
		{
			playState   = PlayState.REGULAR;
		}

		internal static void SwitchToRowRemoval()
		{
			State_02_Row_Removal.Init();
			playState = PlayState.ROW_REMOVAL;
		}

		internal static void SwitchToGameOver()
		{
			State_04_Game_Over.Reset();
			playState = PlayState.GAME_OVER;
		}

	}
}