﻿using System;
using BlockMania.Util;
using System.IO;
using System.Windows.Forms;
using BlockMania.GeneralStates.Screen_02_Play;

namespace BlockMania.GeneralStates.Screen_03_Settings
{
	internal static class SettingStorage
	{
		private static string pathAndFile;

		internal static int hiScoreNew = 0;
		internal static int hiScore = 0;
		internal static int speed = 1;

		internal static bool levelUp = true;
		internal static bool extraBlocks = false;
		internal static bool showGrid = true;

		internal static void ReadSettingsFromDisk()
		{
			pathAndFile = ResourceLocator.pathAndFileNameForSettings;
			if (DoesSettingsFileExist())
			{
				LoadSettings();
				CheckSettings();
			}
		}

		private static bool DoesSettingsFileExist()
		{
			return File.Exists(pathAndFile);
		}

		private static void LoadSettings()
		{
			try
			{
				using (TextReader reader = new StreamReader(pathAndFile))
				{
					string line;
					while ((line = reader.ReadLine()) != null)
					{
						string[] data = line.Split('=');

						string name  = data[0];
						string value = data[1];

						if (!string.IsNullOrEmpty(data[0]))
						{
							switch (name)
							{
								case "hi_score"     : hiScore     = int.Parse(value)  ; break;
								case "speed"        : speed       = int.Parse(value)  ; break;
								case "level_up"     : levelUp     = bool.Parse(value) ; break;
								case "extra_blocks" : extraBlocks = bool.Parse(value) ; break;
								case "show_grid"    : showGrid    = bool.Parse(value) ; break;
							}
						}
					}
				}
			}
			catch (Exception e)
			{
				string msg = $"There seems to be a problem reading file :\n{pathAndFile}\n\n{e.Message}";
				MessageBox.Show(msg, "Cannot read file", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
			}
		}

		private static void CheckSettings()
		{
			if (speed < 0)
			{
				speed = 1;
			}
			else if (speed > BlockSpeed.MAX_SPEED)
			{
				speed = BlockSpeed.MAX_SPEED;
			}
		}

		internal static void SaveSettings()
		{
			if (Screen_3_Settings.settingsAreChanged || hiScoreNew > hiScore)
			{
				string content = $"hi_score={Math.Max(hiScore, hiScoreNew)}\nspeed={speed}\nlevel_up={levelUp}\nextra_blocks={extraBlocks}\nshow_grid={showGrid}";
				SaveTextToDisk(content);
			}
		}

		private static void SaveTextToDisk(string content)
		{
			try
			{
				using (StreamWriter sw = File.CreateText(pathAndFile))
				{
					sw.Write(content);
				}
			}
			catch (Exception e)
			{
				string msg = $"There seems to be a problem writing file :\n{pathAndFile}\n\n{e.Message}";
				MessageBox.Show(msg, "Cannot write file", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
			}
		}
		
	}
}