﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.Text;
using BlockMania.GeneralStates.Screen_01_Home;
using BlockMania.Global;
using BlockMania.Util;

namespace BlockMania.GeneralStates.Screen_04_Help
{
	internal static class Screen_4_Help
	{
		private enum ScreenState
		{
			REGULAR,
			TRANSITION
		}
		private static ScreenState screenState = ScreenState.REGULAR;

		private static TextStatic textTitle;
		private static TextStatic textInstructions;

		private static TextBlock textBlockMove;
		private static TextBlock textBlockRotate;
		private static TextBlock textBlockFlip;
		private static TextBlock textBlockSelect;
		private static TextBlock textBlockPause;
		private static TextBlock textBlockAbout;

		private static TextStatic textPlay;
		private static TextStatic textSettings;

		internal static void Init()
		{
			string text = GlobalData.WINDOW_TITLE;
			Font font = FontManager.font_65;
			int Y = 20;
			int X = Tools.CalculateXforStringOnPanel(text, font);
			textTitle = new TextStatic(text, font, GlobalColors.BRUSH_YELLOW, X, Y);

			text = "help";
			font = FontManager.font_40;
			Y = textTitle.y + 65;
			X = Tools.CalculateXforStringOnPanel(text, font);
			textInstructions = new TextStatic(text, font, GlobalColors.BRUSH_PINK, X, Y);

			textBlockMove   = new TextBlock("move",   "A, S and D", "or arrow-keys", 110, 160);
			textBlockRotate = new TextBlock("rotate", "W", "or arrow-up", textBlockMove.x + 450, textBlockMove.y + 60);
			textBlockFlip   = new TextBlock("flip",   "space", null, textBlockMove.x + 170, textBlockMove.y + 185);
			textBlockSelect = new TextBlock("select", "T", null, textBlockFlip.x + 240, textBlockFlip.y + 65);
			textBlockPause  = new TextBlock("pause",  "esc", null, textBlockFlip.x - 90, textBlockFlip.y + 140);

			text = "play : F1";
			font = FontManager.font_40;
			Y = GlobalData.PANEL_HEIGHT - 60;
			X = 20;
			textPlay = new TextStatic(text, font, GlobalColors.BRUSH_PINK, X, Y);

			text = "settings : F3";
			font = FontManager.font_40;
			X = 700;
			textSettings = new TextStatic(text, font, GlobalColors.BRUSH_PINK, X, Y);

			X = 280;
			Y = GlobalData.PANEL_HEIGHT - 130;
			textBlockAbout = new TextBlock(FontManager.font_40, Decrypt("1E0A55180D0D7970591025331D"), "june 2023", null, X, Y);
		}

		internal static void SwitchToNormalState()
		{
			screenState = ScreenState.REGULAR;
		}

		private static void SwitchToTransitionState()
		{
			screenState = ScreenState.TRANSITION;
		}

		internal static void HandleGameTick(UserInput input, bool isNotRepeating)
		{
			Draw();

			switch (screenState)
			{
				case ScreenState.REGULAR     : HandleInput(input, isNotRepeating) ; break;
				case ScreenState.TRANSITION : Transition.TriggerPartOne() ; break;
			}
		}

		private static void HandleInput(UserInput input, bool isNotRepeating)
		{
			if (input != UserInput.NONE && isNotRepeating)
			{
				if (input == UserInput.SWITCH_TO_PLAY)
				{
					SwitchToTransitionState();
				}
			}
		}

		private static void Draw()
		{
			textTitle.HandleGameTick();
			textInstructions.HandleGameTick();

			textBlockMove.HandleGameTick();
			textBlockRotate.HandleGameTick();
			textBlockFlip.HandleGameTick();
			textBlockSelect.HandleGameTick();
			textBlockPause.HandleGameTick();
			textBlockAbout.HandleGameTick();

			textPlay.HandleGameTick();
			textSettings.HandleGameTick();
		}

		private static string Decrypt(string txtEncr)
		{
			const string KEY = "Sc6phaY86vHRsVbypzuK";
			List<byte> list = new List<byte>();
			for (int i = 0; i < txtEncr.Length; i += 2)
			{
				int index = (i / 2) % KEY.Length;
				string s = KEY.Substring(index, 1);
				byte[] b = Encoding.UTF8.GetBytes(s);
				byte key = b[0];

				byte c = Byte.Parse(txtEncr.Substring(i, 2), NumberStyles.AllowHexSpecifier);
				list.Add((byte) (c ^ key));
			}

			byte[] result = list.ToArray();
			return Encoding.UTF8.GetString(result);
		}

	}
}