﻿using System.Drawing;

namespace BlockMania.Util.BlockTools
{
	internal static class ColorManager
	{
		private  static Color[,]      colorArray;
		internal static SolidBrush[,] brushArray;
		internal static Pen[,]        penArray;

		private const int PERCENTAGE_LIGHTER = 60;
		private const int PERCENTAGE_DARKER  = 20;

		private const int PEN_WIDTH = 1;

		private static readonly int[,] base_colors = {
			 { 255, 255, 255 },
			 { 143, 188, 143 },
			 { 100, 149, 237 },
			 { 218, 165, 32  },
			 { 170, 125, 210 },
			 { 215, 127, 162 },
			 { 213, 155, 123 },
			 { 255, 127, 80  },
			 { 143, 188, 143 },
			 { 100, 149, 237 },
			 { 218, 165, 32  },
			 { 170, 125, 210 },
			 { 215, 127, 162 },
			 { 213, 155, 123 },
			 { 143, 188, 143 },
			 { 100, 149, 237 },
			 { 218, 165, 32  },
			 { 170, 125, 210 },
			 { 215, 127, 162 },
			 { 213, 155, 123 }
		};

		internal static void Init()
		{
			int colorCount = base_colors.Length / 3;

			colorArray = new Color[colorCount, 3];
			brushArray = new SolidBrush[colorCount, 3];
			penArray   = new Pen[colorCount, 3];

			for (int row = 0 ; row < colorCount ; row++)
			{
				Color baseColor = Color.FromArgb(base_colors[row, 0], base_colors[row, 1], base_colors[row, 2]);

				colorArray[row, 0] = MakeLighter(baseColor, PERCENTAGE_LIGHTER);
				colorArray[row, 1] = baseColor;
				colorArray[row, 2] = MakeDarker(baseColor, PERCENTAGE_DARKER);

				brushArray[row, 0] = new SolidBrush(colorArray[row, 0]);
				brushArray[row, 1] = new SolidBrush(colorArray[row, 1]);
				brushArray[row, 2] = new SolidBrush(colorArray[row, 2]);

				penArray[row, 0] = new Pen(colorArray[row, 0], PEN_WIDTH);
				penArray[row, 1] = new Pen(colorArray[row, 1], PEN_WIDTH);
				penArray[row, 2] = new Pen(colorArray[row, 2], PEN_WIDTH);
			}
		}

		private static Color MakeLighter(this Color color, int percentage) // Extension Method.
		{
			return ChangeColorBrightness(color, (float)(percentage / 100.0));
		}

		private static Color MakeDarker(this Color color, int percentage) // Extension Method.
		{
			return ChangeColorBrightness(color, (float)(-1 * percentage / 100.0));
		}

		private static Color ChangeColorBrightness(Color color, float adjustmentFactor)
		{
			// Adjustment-factor must be between -1 and 1, negative values produces darker color.

			float red   = color.R;
			float green = color.G;
			float blue  = color.B;

			if (adjustmentFactor < 0)
			{
				adjustmentFactor = 1 + adjustmentFactor;
				red   *= adjustmentFactor;
				green *= adjustmentFactor;
				blue  *= adjustmentFactor;
			}
			else
			{
				red   = (255 - red)   * adjustmentFactor + red;
				green = (255 - green) * adjustmentFactor + green;
				blue  = (255 - blue)  * adjustmentFactor + blue;
			}

			return Color.FromArgb(color.A, (int) red, (int) green, (int) blue);
		}

	}
}