﻿using System;
using System.Collections.Generic;
using System.Drawing;
using BlockMania.Global;
using BlockMania.Util.BlockTools;

namespace BlockMania.Util
{
	internal static class Tools
	{
		private static readonly Random wheel = new Random();

		internal static int GetRandomNumber(int max)
		{
			return wheel.Next(max); // Returns a number between 0 and 'max', ('max' is exclusive).
		}

		internal static int CalculateXforStringOnPanel(string text, Font font)
		{
			return CalculateXforString(text, font, GlobalData.PANEL_WIDTH, 0);
		}

		internal static int CalculateXforStringOnGamefield(string text, Font font)
		{
			return CalculateXforString(text, font, GlobalData.GAME_FIELD_WIDTH, GlobalData.GAME_FIELD_LOCATION_X);
		}

		internal static int CalculateXforString(string text, Font font, int widthAvailable, int offset)
		{
			SizeF stringSize = GetStringSize(text, font);
			return ( (int) (widthAvailable - stringSize.Width) / 2) + offset;
		}

		internal static SizeF GetStringSize(string text, Font font)
		{
			SizeF stringSize = GlobalCanvas.canvas.MeasureString(text, font);
			return stringSize;
		}

		internal static void DrawRectangleWithBorderAndFill(int x, int y,
			int width, int height, int borderWidth,
			Brush borderBrush, Brush fillBrush)
		{
			// Draw border :
			GlobalCanvas.canvas.FillRectangle(borderBrush, x, y, width, height);

			// Draw fill :
			int xFill = x + borderWidth;
			int yFill = y + borderWidth;

			int widthFill  = width  - (borderWidth * 2);
			int heightFill = height - (borderWidth * 2);

			GlobalCanvas.canvas.FillRectangle(fillBrush, xFill, yFill, widthFill, heightFill);
		}

		internal static void DrawLineHorizontal(Pen pen, int x, int y, int length)
		{
			GlobalCanvas.canvas.DrawLine(pen, x, y, x + length - 1, y);
		}

		internal static void DrawLineVertical(Pen pen, int x, int y, int length)
		{
			GlobalCanvas.canvas.DrawLine(pen, x, y, x, y + length - 1);
		}

		internal static List<List<int>> Copy_2d_List(List<List<int>> listToCopy)
		{
			// Deze method maakt een copy van een 'vierkante' 2d-List.

			var listCopy = new List<List<int>>();
			int listSize = listToCopy[0].Count;

			for (int row = 0 ; row < listSize ; row++)
			{
				var rowList = new List<int>();
				listCopy.Add(rowList);

				for (int col = 0 ; col < listSize ; col++)
				{
					rowList.Add(listToCopy[row][col]);
				}
			}

			return listCopy;
		}

		internal static void DrawTile(int x, int y, int width, int height, int colorCode)
		{
			int colorNumber = (colorCode < 100) ? colorCode : (colorCode - 100);
			bool drawRemovalMark = (colorCode > 100);

			int xTopLeft = x;
			int yTopLeft = y;

			int xBottomRright = xTopLeft + width  - 1;
			int yBottomRight  = yTopLeft + height - 1;

			const int COLOR_LIGHTER = 0;
			const int COLOR_BASE    = 1;
			const int COLOR_DARKER  = 2;

			// Draw body :

			SolidBrush brush = ColorManager.brushArray[colorNumber, COLOR_BASE];
			GlobalCanvas.canvas.FillRectangle(brush, xTopLeft, yTopLeft, width, height);

			// Draw left & top border :

			Pen pen = ColorManager.penArray[colorNumber, COLOR_LIGHTER];
			GlobalCanvas.canvas.DrawLine(pen, xTopLeft, yTopLeft, xTopLeft + width - 1, yTopLeft);
			GlobalCanvas.canvas.DrawLine(pen, xTopLeft, yTopLeft, xTopLeft, yTopLeft + height - 1);

			xTopLeft++;
			yTopLeft++;

			GlobalCanvas.canvas.DrawLine(pen, xTopLeft, yTopLeft, xTopLeft + width - 3, yTopLeft);
			GlobalCanvas.canvas.DrawLine(pen, xTopLeft, yTopLeft, xTopLeft, yTopLeft + height - 3);


			// Draw right & bottom border :

			pen = ColorManager.penArray[colorNumber, COLOR_DARKER];
			GlobalCanvas.canvas.DrawLine(pen, xBottomRright, yBottomRight, xBottomRright, yBottomRight - height + 2);
			GlobalCanvas.canvas.DrawLine(pen, xBottomRright, yBottomRight, xBottomRright - width + 2, yBottomRight);

			xBottomRright--;
			yBottomRight--;

			GlobalCanvas.canvas.DrawLine(pen, xBottomRright, yBottomRight, xBottomRright, yBottomRight - height + 4);
			GlobalCanvas.canvas.DrawLine(pen, xBottomRright, yBottomRight, xBottomRright - width + 4, yBottomRight);

			// Draw removal mark, if appropriate :

			if (drawRemovalMark)
			{
				DrawRectangleWithBorderAndFill(
					x + 6,
					y + 6,
				  width - 12,
					height - 12,
					3,
					brush,
					GlobalColors.BRUSH_BLACK
					);
			}

		}

	}
}