﻿using System.Drawing;
using BlockMania.GeneralStates.Screen_03_Settings;
using BlockMania.Global;
using BlockMania.Util;

namespace BlockMania.View
{
	internal static class Gamefield
	{
		internal static int[,] grid; // This is the Game-field.
		private  static Block block; // This is the Block on the game-field that is controlled by the player.

		private static readonly Pen gridPen = new Pen(GlobalColors.GAME_FIELD_GRID_COLOR, 1);

		internal static void Init()
		{
			Reset();
		}

		internal static void Reset()
		{
			grid = new int[GlobalData.ROW_COUNT, GlobalData.COL_COUNT];
		}

		internal static Block GetBlock()
		{
			return block;
		}

		internal static void SetBlock(Block b)
		{
			block = b;
		}

		internal static bool IsErEenBlockOpHetGamefield()
		{
			return (block != null);
		}

		internal static void MakeBlockPermanent()
		{
			foreach (Tile tile in block.tiles)
			{
				AddTile(tile, block.colorCode);
			}

			block = null;
		}

		internal static bool IsLocationOccupied(int row, int col)
		{
			return (grid[row, col] != 0);
		}

		internal static void Render()
		{
			DrawGameField();
			DrawGrid();
			DrawPermanentTiles();
			DrawBlockTiles();
		}

		private static void DrawGameField()
		{
			const int x = GlobalData.GAME_FIELD_LOCATION_X;
			const int y = GlobalData.GAME_FIELD_LOCATION_Y;

			const int width  = GlobalData.GAME_FIELD_WIDTH;
			const int height = GlobalData.GAME_FIELD_HEIGHT;

			const int borderWidth = GlobalData.GAME_FIELD_BORDER_WIDTH;

			Brush borderColor = GlobalColors.GAME_FIELD_BORDER_COLOR;
			Brush fieldColor  = GlobalColors.GAME_FIELD_COLOR;

			Tools.DrawRectangleWithBorderAndFill(x, y, width, height,
				borderWidth, borderColor, fieldColor);
		}

		private static void DrawGrid()
		{
			if (SettingStorage.showGrid)
			{
				DrawHorizontalGridLines();
				DrawVerticalGridLines();
			}
		}

		private static void DrawHorizontalGridLines()
		{
			const int borderWidth = GlobalData.GAME_FIELD_BORDER_WIDTH;
			const int x = GlobalData.GAME_FIELD_LOCATION_X + borderWidth;
			const int length = GlobalData.GAME_FIELD_WIDTH - (borderWidth * 2);

			for (int row = 1 ; row < GlobalData.ROW_COUNT ; row++)
			{
				int y = GlobalData.GAME_FIELD_LOCATION_Y + (row * GlobalData.TILE_HEIGHT);
				Tools.DrawLineHorizontal(gridPen, x, y, length);
			}
		}

		private static void DrawVerticalGridLines()
		{
			const int borderWidth = GlobalData.GAME_FIELD_BORDER_WIDTH;
			const int y = GlobalData.GAME_FIELD_LOCATION_Y + borderWidth;
			const int length = GlobalData.GAME_FIELD_HEIGHT - (borderWidth * 2);

			for (int col = 1 ; col < GlobalData.COL_COUNT ; col++)
			{
				int x = GlobalData.GAME_FIELD_LOCATION_X + (col * GlobalData.TILE_WIDTH);
				Tools.DrawLineVertical(gridPen, x, y, length);
			}
		}

		private static void DrawPermanentTiles()
		{
			for (int row = 0; row < GlobalData.ROW_COUNT ; row++)
			{
				for (int col = 0; col < GlobalData.COL_COUNT ; col++)
				{
					int colorCode = grid[row,col];
					if (colorCode > 0)
						DrawTile(row, col, colorCode);
				}
			}
		}

		private static void DrawTile(int row, int col, int colorCode)
		{
			int x = (col * GlobalData.TILE_WIDTH)  + GlobalData.GAME_FIELD_LOCATION_X;
			int y = (row * GlobalData.TILE_HEIGHT) + GlobalData.GAME_FIELD_LOCATION_Y;

			const int width  = GlobalData.TILE_WIDTH;
			const int height = GlobalData.TILE_HEIGHT;

			Tools.DrawTile(x, y, width, height, colorCode);
		}

		private static void DrawBlockTiles()
		{
			if (block != null)
			{
				foreach (Tile tile in block.tiles)
				{
					DrawBlockTile(tile, block.colorCode);
				}
			}
		}

		private static void DrawBlockTile(Tile tile, int colorCode)
		{
			DrawTile(tile.row, tile.col, colorCode);
		}

		private static void AddTile(Tile tile, int colorCode)
		{
			grid[tile.row, tile.col] = colorCode;
		}

		internal static void RemoveTile(int row, int col)
		{
			grid[row, col] = 0;
		}

	}
}